#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "ob_inter.h"

#define HASHSIZE 101
#define MIN_INTERVAL_TIME  (tres*1000)

int ob_run_fortran = 0;
#include "scamem.h"
#include "scahpt.h"

extern void oborg_forcecpyfunction( int n );

static int failsExpectedStop = 0;

static Interval *itab[HASHSIZE];

static Interval *currentInterval = NULL;
static int stop = 0;

static char *fname;
static int rank;
static int nofIntervals;
static int **flowGraph;
static char **flowGraphElements;
static int flowGraphIdx=-1;
static int flowGraphPreviousIdx=-1;
static int logSize = 64;
static int noAnneal = 0;
static int doContinue = 0;
static double min_interval_time;
static double tres;

static void findSelectionRange( Info *, int, int);
static void readFile(void);
static void put( Interval *);
static unsigned hash( char *);
static Interval *lookupInterval( char *);
static int isExpectedStop( char *, Interval *, int *);
static void setvalue( Info *);
static void writeFile(void);
static void createLog(void);
static void init_selection_range(void); 
static void printInterval(Interval *);
static void printAllIntervals(void);

void ob_registerInLog(char *);
void ob_noop(char *);
void ob_inter(char *);

extern void printStatistics(void);

/*--------------------------------------------------------*/
/* For statistics */
/*--------------------------------------------------------*/

static int statistics_over_limit = 0;
static int statistics_accumulated= 0;
static int statistics_total_accumulated = 0;

/** Preliminary selection ranges **/

static int *scamemcpy_select_all_table;
static int scamemcpy_select_all_length = 35;

static void
printAllIntervals(void){
   Interval *iv = NULL;
   int i,idx = 0;
   
   while(!iv){
      iv = itab[idx++];
   }

   for( i = 0; i < nofIntervals ; i ++ ){
      printInterval( iv );
      // Find a new interval
      if( ! (iv->nextinhash))
	 while((iv = itab[idx++]) == NULL);
      else
	 iv = iv->nextinhash;
   }
}


static void
printInterval( Interval *iv ){
   int i;
   char *hline = "--------------------------------------------------\n";
   printf("Target  : %s\n", iv->target);
   printf("Final   : %s\n", iv->final);
   printf("ilength : %d\n", iv->ilength);
   printf("Temp    : %.3f\n", iv->temp);
   printf("Trials  : %d\n\n", iv->trials);
   
   printf("NofStop : %d\n", iv->nofStopAdrs);
   for( i = 0 ; i < iv -> nofStopAdrs ; i ++ )
      printf("%s\n", iv->stopAdr[i]);
   printf("\n\n");

   printf("Members:\n");
   for( i = 0; i < iv->ilength ; i ++ ){
      Info *ifo;
      int k;
      ifo = iv -> info[i];
      printf("%s",hline);
      printf("Adr: %s\n",ifo->fullAddress);
      for( k = 0 ; k < ifo->selectionLength ; k ++ ){
	 printf(" %d , %f \n", ifo->selection[k], ifo->relcost[k]);
      }
   }   
}


/*----------------------------------------------------------------*/

static void
init_selection_range(void){
   int i,n = scamemcpy_select_all_length;

   scamemcpy_select_all_table = (int*)malloc( n * sizeof(int));

   for( i = 0; i < n ; i ++ ){
      scamemcpy_select_all_table[i] = i +1 ;
   }
}



/*******************************************************************/
/* THE LOG PART */
/*******************************************************************/

static void
createLog(void){
   char *env;
   int i;
   

   if( (env = getenv("OB_MAX_LOG_SIZE")) != NULL ){
      logSize = atoi(env);
   }

   flowGraph = (int **)malloc( logSize * sizeof(int *));
   flowGraph[0] = (int *)malloc( logSize * logSize * sizeof(int));

   for( i = 1; i < logSize ; i ++ ){
      flowGraph[i] = flowGraph[i-1] + logSize;
   }
   for( i = 0; i < (logSize*logSize) ; i ++ ){
      flowGraph[0][i] = 0;
   }

   flowGraphElements = (char **)malloc( logSize * sizeof(char *));
}

/*******************************************************************/


void
ob_registerInLog(char *signature){
   int i;
   char *newElement;

   for( i = 0; i <=  flowGraphIdx ; i ++ ){
      if( STREQUALS( signature , flowGraphElements[i] ) ){
	 flowGraph[flowGraphPreviousIdx][i] ++;
	 flowGraphPreviousIdx = i;
	 return;
      } 
   }

   if((++flowGraphIdx) >= logSize ){
      printf("Too many registrated addresses, try with a larger ");
      printf("value of OB_MAX_LOG_SIZE\n");
      exit(1);
   }
	  	  
   newElement = (char *)malloc((strlen(signature)+1) * sizeof(char));
   strcpy( newElement, signature );
   flowGraphElements[flowGraphIdx] = newElement;

   if( flowGraphPreviousIdx != -1 )
      flowGraph[flowGraphPreviousIdx][flowGraphIdx] ++ ;

   flowGraphPreviousIdx = flowGraphIdx;
}

/*******************************************************************/

static void
printLog(void){
   int i,j;
   char *env;
   FILE *fp;
   char outFileName[64];

   if( (env = getenv("OB_WRITE_FILE")) != NULL){
      sprintf( outFileName, "%s_%d", env, rank );
      fp = fopen( outFileName, "w");
      
      fprintf( fp, "%d\n", flowGraphIdx + 1 );
      for( i = 0; i <= flowGraphIdx ; i ++ ){
	 fprintf(fp, "%s\n", flowGraphElements[i]);
      }
      
      for( i = 0; i <= flowGraphIdx ; i ++ ){
	 for( j = 0; j <= flowGraphIdx ; j ++ ){
	    fprintf(fp, " %d ", flowGraph[i][j]);
	 }
	 fprintf(fp,"\n");
      }
   }
   else{
      printf("%d\n", flowGraphIdx + 1 );
      for( i = 0; i <= flowGraphIdx ; i ++ ){
	 printf("%s\n", flowGraphElements[i]);
      }
      
      for( i = 0; i <= flowGraphIdx ; i ++ ){
	 for( j = 0; j <= flowGraphIdx ; j ++ ){
	    printf(" %d ", flowGraph[i][j]);
	 }
	 printf("\n");
      }
   }
}



/*******************************************************************/
/* THE SETUP PART */
/*******************************************************************/


void
ob_inter_init(void){
   char *env;
   
   {
      char progName[200];
      char subMonitorName[200];
      int subMonitorPort;
      int globalRank;
      char monitorName[200];
      int monitorPid;
      char* scampiProcessParam = getenv("SCAMPI_PROCESS_PARAM");
      sscanf(scampiProcessParam,"%200s %200s %d %d %200s %d",
	     progName,
	     subMonitorName,
	     &subMonitorPort,
	     &globalRank,
	     monitorName,
	     &monitorPid);
      rank  = globalRank;
   }


   init_selection_range();

   if( (env = getenv("OB_CONTINUE")) != NULL ){
      doContinue = 1;
   }

   if( (env = getenv("OB_READFILE")) != NULL ){
      fname = (char*)malloc( (strlen(env)+10) * sizeof(char));
      sprintf( fname, "%s_%d", env, rank );
      readFile();
      ob_interval = ob_inter;
   }
   else if( (env = getenv("OB_CONSTRUCT")) != NULL ){
      createLog();
      ob_interval = ob_registerInLog;
   }
   else{
      ob_interval = ob_noop;
   }


   if( (env = getenv("OB_NOANNEAL")) != NULL ){
      noAnneal = 1;
   }


   {
      double t1,t2;
      tres = 1.0;
      int k;
      for( k = 0; k < 5 ; k ++ ){
	 for( t1 = hptGetSecs(); (t2 = hptGetSecs()) == t1 ;);
	 
	 if(tres > (t2 - t1)) tres = t2 - t1 ;
      }
   }

   if( (env = getenv("OB_MIN_INTERVAL_TIME"))){
      min_interval_time = (double)atoi(env) * tres;
   }
   else{
      min_interval_time = MIN_INTERVAL_TIME ;
   }


   ob_sao_init();

   if( (env = getenv("OB_VERBOSE_INIT")) != NULL ){
      printAllIntervals();
   }

   printf("Rank %d, finnished init\n", rank );
}

/*******************************************************************/
void
ob_inter_final(void){

   if( ob_interval == ob_inter ){
      char *env;
      writeFile();

      if( (env = getenv("OB_STATISTICS")) != NULL){

	 printStatistics();
	 printf("Over the limit: %d\n", statistics_over_limit );
	 printf("Accumulated : %d\n", statistics_accumulated );
	 printf("Total acc : %d\n", statistics_total_accumulated );
	 printf("------------------------------------\n");
  
	 printAllIntervals();
      }
   }   
   else if( ob_interval == ob_registerInLog ){
      printLog();
   }
}




/*******************************************************************/

static void
findSelectionRange(Info *ifo, int typecode, int size ){
   /* This function is used to reduce the solution space by
    * discriminating on the typecode. */
   
   int blen = scamemcpy_select_all_length * sizeof(int);
   ifo -> selectionLength = scamemcpy_select_all_length;
   ifo -> selection = (int*)malloc(blen);
   memcpy( ifo -> selection, scamemcpy_select_all_table,blen);
}

/*******************************************************************/
/* THE INTERVAL PART*/
/*******************************************************************/

static void
put( Interval *iv ){
   unsigned hashval;
   if( lookupInterval( iv->target ) == NULL ){
      hashval = hash( iv->target );
      iv -> nextinhash = itab[hashval];
      itab[hashval] = iv;
   }
   else{
      printf("*** ERROR ***\n");
      printf("Multiple intervals with common target: %s\n", iv -> target);
      exit(1);
   }
}

/*******************************************************************/

static unsigned
hash( char *s ){
  unsigned hashval;
  for ( hashval = 0; *s != '\0' ; s++ )
    hashval = *s + 31 * hashval ;
  return hashval % HASHSIZE ;
}

/*******************************************************************/

static Interval *
lookupInterval( char *s ){
   Interval *iv;
   for( iv = itab[ hash(s) ] ; iv != NULL; iv = iv -> nextinhash )
      if( strcmp( s, iv -> target ) == 0 )
	 return iv;
   return NULL;
}

/*******************************************************************/

static int
isExpectedStop( char *s , Interval *iv, int *idx ){
   int i;
   for( i = 0; i < iv->nofStopAdrs ; i++ ){
      if(STREQUALS( s, iv->stopAdr[i] )){
	 *idx = i;
	 return 1;
      }
   }
   *idx = -1; //for testing only
   return 0;
}

/*******************************************************************/

static void
setvalue( Info *ifo ){
   //  const int mask = OB_SCAMEMFORCEFUNCTION | OB_GATHER | OB_BCAST ;
   oborg_forcecpyfunction( ifo->selection[ifo->cidx] );
/*

   switch (ifo->typecode & mask){

   case OB_SCAMEMFORCEFUNCTION:
      oborg_forcecpyfunction( ifo->cidx );
      break;
   case OB_GATHER:
      printf("ob_forcegatherfunc( ifo->cidx );\n");
      break;
   case OB_BCAST:
      printf("ob_forcebcastfunc( ifo->cidx );\n");
   }
*/
}



/*******************************************************************/

void
ob_inter( char *returnAddress ){

   if( currentInterval == NULL ){
      if( ! (currentInterval = lookupInterval( returnAddress ))){
	 return;
      }
   }
   

   if( stop ){
      int idx;
      double t;
      stop = 0;
      if ( !noAnneal){
	 if likely( isExpectedStop( returnAddress , currentInterval, &idx )){
	    t = hptGetSecs() - currentInterval -> startTime;
	    if( t > min_interval_time ){
	       statistics_over_limit ++;
	       ob_simAnnealOnce( currentInterval, t, idx );
	    }
	    else if( (currentInterval->accumulativeTime += t) > min_interval_time ){
	       t = currentInterval->accumulativeTime /\
		  (++currentInterval->nofAccumulative);
	       ob_simAnnealOnce( currentInterval, t, idx );
	       statistics_total_accumulated += currentInterval->nofAccumulative;
	       currentInterval->nofAccumulative = 0;
	       currentInterval->accumulativeTime = 0.0;
	       statistics_accumulated ++;
	    }
	    else{
	       currentInterval -> nofAccumulative ++ ;
	    }
	 }
	 else{
	    failsExpectedStop ++;
	 }
      }
      if(!STREQUALS( currentInterval->target, returnAddress ))
	 if( ! (currentInterval = lookupInterval( returnAddress ))){
	    return;
	 }
   }


   if( STREQUALS( currentInterval->target, returnAddress ) ){
      currentInterval->startTime = hptGetSecs();
      currentInterval-> idx = 0;
   }

   
   if( STREQUALS( currentInterval->final, returnAddress ) ){
      stop = 1;
   }


   setvalue( currentInterval -> info[ (currentInterval-> idx) ++ ] );

}

void 
ob_noop(char *dontcare){
   return;
}

/*******************************************************************/
/* THE IO PART */
/*******************************************************************/


static void
readFile(void){
   FILE *fp;      
   int nofMembers;
   int nofStopAdrs;
   int nofSelection;
   int trials;
   float temp;
   char address[32];
   char typecode[32];
   char size[32];
   char selection[32];
   char fullAddress[64];
   int i,j,k;
   
   fp = fopen( fname, "r");
   fscanf( fp, "%d", &nofIntervals );
   
   /* Loop over all intervals */
   
   for( i = 0; i < nofIntervals ; i ++ ){
      Interval *iv;
      iv = malloc(sizeof(*iv));
      
      fscanf( fp, "%d", &nofMembers );
      iv -> ilength = nofMembers;
      iv -> info = (Info**)malloc( nofMembers * sizeof(Info*));
      
      
      fscanf( fp, "%f", &temp );
      fscanf( fp, "%d", &trials);

      /* Loop over members in the interval */

      for( j = 0; j < nofMembers ; j ++ ){
	 Info *ifo;
	 fscanf( fp, "%s", address  );
	 fscanf( fp, "%s", typecode );
	 fscanf( fp, "%s", size     );
	 fscanf( fp, "%s", selection);
      
	 sprintf( fullAddress, "%s %s %s", address, typecode, size );
	 
	 if( j == 0 )
	    strcpy( iv -> target, fullAddress );
	 if( j == ( nofMembers - 1 ))
	    strcpy( iv -> final , fullAddress );

	 
	 ifo = (Info*)malloc(sizeof(Info));
	 ifo -> typecode = atoi(typecode);
	 strcpy(ifo -> fullAddress, fullAddress );

	 /* If nofSelection is 0, this is a "startup" file */

	 if((nofSelection = atoi(selection)) > 0 ){
	    ifo -> selectionLength = nofSelection;
	    ifo -> selection = (int*)malloc( nofSelection * sizeof(int));
	    for( k = 0; k < nofSelection ; k ++ ){
	       fscanf( fp, "%s", selection );
	       ifo -> selection[k] = atoi(selection);
	    }
	 }
	 else{
	    findSelectionRange(ifo , atoi(typecode), atoi(size));
	 }
	 
	 ifo -> relcost=(double*)calloc(ifo->selectionLength, sizeof(double));
	 ifo -> aidx = ifo -> bidx = ifo -> cidx = 0; 

	 iv -> info[j] = ifo;
      } /* End looping over the members */

      fscanf( fp, "%d", &nofStopAdrs );
      iv -> stopAdr = ( char**)malloc( nofStopAdrs );
      iv -> times   = (double*)malloc( nofStopAdrs * sizeof(double));
      iv -> btimes  = (double*)malloc( nofStopAdrs * sizeof(double));
      iv -> nofStopAdrs = nofStopAdrs;
      iv -> nofAccumulative = 0;
      iv -> accumulativeTime = 0.00;
      iv -> idx = 0;
      iv -> firstIteration = 1;


      if( doContinue ){
	 iv -> temp = (double)temp;
	 iv -> trials = trials;
      }
      else{
	 iv -> temp = 1.000;
	 iv -> trials = 0;
      }

      /* Loop over stop addresses */
      for( j = 0; j < nofStopAdrs ; j ++ ){
	 fscanf( fp, "%s", address );
	 fscanf( fp, "%s", typecode );
	 fscanf( fp, "%s", size );
	 sprintf( fullAddress, "%s %s %s", address, typecode, size );	 
	 
	 iv -> stopAdr[j] = (char*)malloc( (strlen(fullAddress)+1));
	 strcpy( iv -> stopAdr[j] , fullAddress );
	 
	 iv -> btimes[j] = MAX_TIME ;
      }
      put(iv);
   } /* End looping over the intervals */
}

/*******************************************************************/

static void 
writeFile(void){
   FILE *fp;
   int nofMembers;
   int nofStopAdrs;
   int i,j,k,idx=0;
   Interval *iv = NULL;

   
   /* Find an interval */
   while( !iv ){	
      iv = itab[idx++];
   }  

   fp = fopen( fname, "w" );
   fprintf( fp, "%d\n", nofIntervals );
   /* Loop over all intervals */
   for( i = 0; i < nofIntervals ; i ++ ){
      nofMembers = iv -> ilength;
      fprintf( fp, "%d\n", nofMembers);
      fprintf( fp, "%f\n", iv -> temp );
      fprintf( fp, "%d\n", iv -> trials );
      /* Loop over all members and write their selections */
      for( j = 0; j < nofMembers; j ++ ){
	 Info *ifo = iv->info[j];
	 fprintf(fp, "%s %d", ifo -> fullAddress, ifo -> selectionLength);
	 
	 for( k = 0; k < ifo -> selectionLength ; k ++ ){
	    fprintf(fp, " %d", ifo -> selection[k]);
	 }

	 fprintf(fp, "\n" );
      }
      
      nofStopAdrs = iv -> nofStopAdrs;
      fprintf( fp, "%d\n", nofStopAdrs );
      for( j = 0; j < nofStopAdrs; j ++ ){
	 fprintf( fp, "%s\n" ,iv -> stopAdr[j] );
      }     

      // Find a new interval
      if( ! (iv->nextinhash))
	 while((iv = itab[idx++]) == NULL);
      else
	 iv = iv->nextinhash;
   }

}

/*******************************************************************/
