import java.util.*;
import java.io.*;


public class Identify{

   public static boolean verbose = true;
   public static boolean printloop = true;
   public static boolean iverbose = true;
   public static boolean output = false;

   public ArrayList allloops;

   public Identify(){
      allloops = new ArrayList();
   }


   public void mainLoop( Node[] djGraph ){

      Arrays.sort( djGraph, new DominTreeLevelComparator() );


      int previousLevel = djGraph[0].getDominTreeLevel();
      boolean irreducibleLoop = false;
      
      
      for( int i = 0; i < djGraph.length; i ++ ){
	 Node n = djGraph[i];

	 if( n.getDominTreeLevel() < previousLevel ){
	    /* SCC */
	    if(verbose)System.out.println("New domintreelevel");
	    if( irreducibleLoop ){
	       if(verbose)System.out.println("Starting on irreducible loop");
	       clearFound( djGraph );
	       int nof = 0;
	       for( int j = 0 ; j < i ; j ++ ){
		  if( !djGraph[j].getIsContained() )
		     nof ++;
	       }

	       Node[] tmp = new Node[nof];
	       for( int j = i -1; j > -1 ; j -- ){
		  if( !djGraph[j].getIsContained() )
		     tmp[--nof] = djGraph[j];
	       }

	       if(verbose){
		  System.out.println("Considered nodes : ");
		  for( int ii = 0; ii < tmp.length ; ii ++ ){
		     System.out.print(tmp[ii].getSignature() +" ");
		  }
		  System.out.print("\n");
	       }
	       
	       dfslevel( tmp, previousLevel );
	       Arrays.sort( tmp, new FnumberComparator(false) );
	       clearFound( tmp );
	       ArrayList scc = dfsscc( tmp , previousLevel );
	       if(verbose)System.out.println("Irreducible loops");

	       for( int j = 0; j < scc.size() ; j ++ ){
		  Node[] sloop = (Node[])scc.get(j);
		  if( sloop.length < 2 ) continue;
		  allloops.add( sloop );
		  if( printloop ){
		     System.out.print("[");
		     for( int jj = 0; jj < sloop.length ; jj ++ ){
			System.out.print(" "+sloop[jj].getSignature() );   
		     }
		     System.out.print(" ]\n");
		  }
	       }

	       for( int j = 0; j < scc.size() ; j ++ ){
		  collapseNode( (Node[])scc.get(j) );
	       }
	    }
	    irreducibleLoop = false;
	    previousLevel = n.getDominTreeLevel();
	 }


	 if(verbose)System.out.println("Node "+n.getSignature()+" started");

	 if( n.getIsContained() ){
	    if(verbose)System.out.println("Continued #####################");
	    continue;
	 }


	 /*
	   Here begins the search for the reducible loops.
	   Three different problems emerge:

	   1) The article does not state whether 
	   we seach for bj or sp-back edges.

           2) When two loops share the first n nodes, but then
	   diverge, the loops should be recorded before collaption.

	   3) When node N is a target for two or more loops, 
	   and one of them is contained within the other, 
	   the inner loop should be collapsed first. This 
	   happends, but the early recording of loops 
	   in (2) return erroneous loops.
              
	   Solution to 2 and 3 is to search for all reachundersets
	   and determine if any of them are subsets of the others.

	 */
			
	 EdgeList el = n.getInEdges();
	 boolean foundreducible = false;
	 for( int j = 0 ; j < el.size() ; j ++ ){
	    if( !irreducibleLoop )
	       irreducibleLoop = el.get(j).isIrrEdge();
	    if( el.get(j).isBJEdge() ){
	       if( ! foundreducible ){
		  foundreducible = true;
		  ArrayList rsets = new ArrayList();
		  for( int jj = j ; jj < el.size() ; jj ++ ){
		     if( el.get(jj).isBJEdge() ){
			Node src = el.get(jj).getSrc();
			ArrayList set = src.reachUnderSet(n);
			if( set.size() > 0 )
		  	   rsets.add(set );
		     }
		  }
		  ArrayList[] fsets = new ArrayList[rsets.size()];
		  fsets = (ArrayList[])rsets.toArray( fsets );
		  for( int ii = 0; ii < fsets.length ; ii ++ ){
		     if(printloop)System.out.print("[ "+n.getSignature());
		     Node[] rloop = new Node[fsets[ii].size()+1];
		     rloop[0] = n;
		     for( int kk = 0; kk < fsets[ii].size() ; kk ++ ){
			if(printloop)System.out.print(" "+ ((Node)(fsets[ii].get(kk))).getSignature());
			rloop[kk+1] = (Node)(fsets[ii].get(kk));
		     }
		     if(printloop)System.out.print(" ]\n");
		     allloops.add(rloop);
		  }	       
		  Node src = el.get(j).getSrc();	       
		  collapseNode( src.reachUnderSet(n) , n );
		  el = n.getInEdges();
		  j = 0;
	       }
	    }
	 }
      }
   }

   private ArrayList[] reduceSets( ArrayList rsets ){

      ArrayList[] fsets = new ArrayList[rsets.size()];
      rsets.toArray( fsets );
      Arrays.sort( fsets, new AListComparator() );

      for( int i = 0; i < fsets.length -1 ; i ++ ){
	 ArrayList a = fsets[i];
	 for( int j = i +1 ; j < fsets.length ; j ++ ){
	    ArrayList b = fsets[j];
	    clean( a, b );
	 }
      }
      return fsets;
   }

   private void clean( ArrayList a, ArrayList b ){
      if( !realSubSet( a, b ) )return;
      for( int i = 0; i < a.size() ; i ++ ){
	 for( int j = 0; j < b.size() ; j ++ ){
	    if( a.get(i) == b.get(j) ){
	       b.remove(j--);
	    }
	 }	    
      }
   }
   
   private boolean realSubSet( ArrayList a, ArrayList b ){
      int matches = 0;
      for( int i = 0; i < a.size() ; i ++ ){
	 for( int j = 0; j < b.size() ; j ++ ){
	    if( a.get(i) == b.get(j)){
	       matches ++ ;
	       break;
	    }
	 }
      }
      return matches == a.size();
   }

   public void collapseNode( Node[] nl ){
      Arrays.sort( nl, new DominTreeLevelComparator() );
      ArrayList al = new ArrayList();
      for( int i = 0; i < nl.length -1 ; i ++ ){
	 al.add( nl[i] );
      }
      collapseNode( al , nl[nl.length - 1 ] );
   }

   public void collapseNode( ArrayList rset, Node n ){
      if( rset.size() == 0 )
	 return;
      
      if(verbose){
	 System.out.println("Collapsing nodes :");
	 for( int i =0 ; i < rset.size() ; i ++ ){
	    System.out.print( ((Node)rset.get(i)).getSignature() +" | " );
	 }
	 System.out.print("\nWith "+n.getSignature()+" as a target\n");
      }

      EdgeList inel = n.getInEdges();
      EdgeList outel = n.getOutEdges();

      EdgeList fin  = new EdgeList();
      EdgeList fout = new EdgeList();

      for( int i = 0; i < rset.size() ; i ++ ){
	 Node inner = (Node)rset.get(i);
	 inel.extend( inner.getInEdges() );
	 outel.extend( inner.getOutEdges() );
      }
      
      for( int i = 0; i < inel.size() ; i ++ ){
	 Node src = inel.get(i).getSrc();
	 if( ! rset.contains( src ) && src != n ){
	    inel.get(i).setDst( n );
	    fin.add( inel.get(i) );
	 }
      }

      for( int i = 0; i < outel.size() ; i ++ ){
	 Node dst = outel.get(i).getDst();
	 if( !rset.contains( dst ) && dst != n ){
	    outel.get(i).setSrc( n );
	    fout.add( outel.get(i));
	 }
      }

      for( int i = 0; i < rset.size() ; i ++ ){
	 ((Node)( rset.get(i))).setIsContained( true );
      }
      
      n.setInEdges( fin );
      n.setOutEdges( fout );

      if(verbose){
	 System.out.println("After collapsing "+n.getSignature()+" it as these properties:");
	 System.out.println( n );
      }

   }


   public void buildDJGraph( Node[] dominTree, Node start ){

      for( int i = 0; i < dominTree.length; i ++ ){
	 if( !dominTree[i].hasStrParent( dominTree[i].getIdom() ) ){
	    Edge e = new Edge( dominTree[i].getIdom() , dominTree[i] );
	    dominTree[i].addInEdge( e );
	    dominTree[i].getIdom().addOutEdge( e ) ;
	 }
      }
      
      for( int i = 0; i < dominTree.length ; i ++ ){
	 dominTree[i].setDominTreeLevel();
      }
      
      clearFound( dominTree );
      dfs( start );
      clearFound( dominTree );

      for( int i = 0; i < dominTree.length ; i ++ ){
	 dominTree[i].setOutEdgeCharacteristics();
      }      
   }

   private void clearFound( Node[] nl ){
      for( int i = 0; i < nl.length; i ++ ){
	 nl[i].setFound( false );
      }
   }

   public void buildDominTree( Node[] flowgraph, Node start ){
      start.setIdom( start );
      dfs( flowgraph  );
      Arrays.sort( flowgraph, new FnumberComparator(true));
    
      boolean changed = true;
      while( changed ){
	 changed = false;
	 for( int i = flowgraph.length - 2 ; i > -1 ; i-- ){
	    Node n = flowgraph[i];
	    EdgeList el = n.getInEdges();
	    int k = 0;
	    Node newIdom = el.get( k ++ ).getSrc() ;
	    while( newIdom.getIdom() == null )
	       newIdom = el.get( k ++ ).getSrc() ;
	    
	    for( ; k < el.size(); k ++ ){
	       Node tmp = el.get( k ).getSrc();
	       if( tmp.getIdom() != null )
		  newIdom = intersect( newIdom, tmp );
	    }
	    if( n.getIdom() != newIdom ){
	       changed = true;
	       n.setIdom( newIdom );
	    }
	    
	 }
      }
   }

   private Node intersect( Node b1, Node b2 ){
      while( b1 != b2 ){
	 while( b1.getFnumber() < b2.getFnumber() )
	    b1 = b1.getIdom();
	 while( b2.getFnumber() < b1.getFnumber() )
	    b2 = b2.getIdom();
      }
      return b1;
   }

   public void dfs( Node[] nodelist ){
      int fnr = 0;
      for( int i = 0; i < nodelist.length ; i ++ ){
	 if( !nodelist[i].getFound() )
	    fnr =dfsvisit( nodelist[i], nodelist[i], fnr );
      }
   }

   public void dfs( Node start  ){
      int fnr = 0; 
      dfsvisit( start, start,  fnr );
   }


   public int dfsvisit( Node n, Node p, int fnr ){
      EdgeList el;
      n.setFound( true );
      el = n.getOutEdges();	
      
      for( int i = 0; i < el.size() ; i ++ ){
	 if( ! el.get(i).getDst().getFound() )
	    fnr = dfsvisit( el.get( i ).getDst(), n,  fnr );	 
      }
      
      n.setPre( p );
      n.setFnumber( fnr ++ );
      return fnr;
   }

   public void dfslevel( Node[] nl, int level ){
      int fnr = 0;
      for( int i = 0; i < nl.length; i ++ ){
	 if( !nl[i].getFound() && nl[i].getDominTreeLevel() >= level)
	    fnr = dfsvisitlevel( nl[i], level, fnr );
      }
   }
   
   public int dfsvisitlevel( Node n, int level, int fnr ){
      EdgeList el = n.getOutEdges();
      n.setFound( true );
      for( int i = 0; i < el.size() ; i ++ ){
	 Node dst = el.get(i).getDst();
	 if( ! dst.getFound() ){
	    if( dst.getDominTreeLevel() >= level ){
	       dst.setFound( true );
	       fnr = dfsvisitlevel( dst , level, fnr );
	    }
	 }
      }
      
      n.setFnumber( fnr ++ );
      return fnr;
   }

   public ArrayList dfsscc( Node[] nl, int level ){
      ArrayList ret = new ArrayList();
      for( int i = 0; i < nl.length ; i ++ ){
	 if( ! nl[i].getFound() && nl[i].getDominTreeLevel() >= level ){
	    ArrayList scc = dfsvisitscc( nl[i], level );	 
	    Node[] tmp = new Node[scc.size()];
	    ret.add( scc.toArray( tmp ) );
	 }
      }
      return ret;
   }
   
   public ArrayList dfsvisitscc( Node n , int level ){
      EdgeList el = n.getInEdges();
      ArrayList found = new ArrayList();
      n.setFound( true );
      for( int i = 0; i < el.size() ; i ++ ){
	 Node src = el.get(i).getSrc();
	 if( ! src.getFound() && src.getDominTreeLevel() >= level ){
	    found = union( found, dfsvisitscc( src , level ) );
	 }
      }
      found.add( n );
      return found;
   }

   private ArrayList union( ArrayList a, ArrayList b ){
      ArrayList ret = new ArrayList();
      ret.addAll( b );
      for( int i = 0; i < a.size() ; i ++ ){
	 if( ! ret.contains( a.get( i ) ) )
	    ret.add( a.get(i));
      }
      return ret;
   }


   public void printLoops(){
      for( int i = 0; i < allloops.size() ; i ++ ){
	 Node[] na = (Node[])allloops.get(i);
	 System.out.print("[");
	 for(int j = 0; j < na.length; j ++ ){
	    System.out.print(" "+na[j].getSignature()+" ");
	 }
	 System.out.print("]\n");
      }
   }


   public void printIntervals( ){
      ArrayList intervals = new ArrayList();
      Node[] targets = new Node[allloops.size()];

      //Step 1.
      if(iverbose)System.out.println("Step1");
      for( int i = 0; i < allloops.size() ; i ++ ){
	 Node[] na = (Node[])allloops.get(i);
	 targets[i] = na[0];
	 if( na.length == 2 ){
	    if( na[0] == na[1] ){
	       na = new Node[1];
	       na[0] = targets[i];
	    }
	 }
	 intervals.add( new Interval(na));
      }
      
      


      
      // Step 2.
      if(iverbose)System.out.println("Step2");
      ArrayList tmpintervals = (ArrayList)intervals.clone();
      intervals.clear();
      while( ! tmpintervals.isEmpty() ){
	 Interval iv = (Interval)(tmpintervals.remove(0));
	 boolean changed = false;
	 for( int i = 0; i < targets.length ; i ++ ){
	    Interval[] iva = iv.split( targets[i] );
	    if( iva.length > 1 || iva[0] != iv ){
	       for( int j = 0 ; j < iva.length ; j ++ ){
		  tmpintervals.add( iva[j] );
	       }
	       changed = true;
	       break;
	    }
	 }
	 if( !changed ){
	    intervals.add(iv);
	 }
      }

      // Step 3. and 4.
      if(iverbose)System.out.println("Step3,4");
      intervals = sortAndRemoveDuplicates( intervals );
      
      if(iverbose){
	 System.out.println("Before step 5 : ");
	 for( int i = 0; i < intervals.size() ; i ++ ){
	    System.out.println( (Interval)(intervals.get(i)));
	 }
      }
      
      // Step 5.
      if(iverbose)System.out.println("Step5");
      boolean changed = true;
      while( changed ){
	 Interval ia = (Interval)(intervals.get(0));;
	 Interval ib = ia;
	 tmpintervals.clear();
	 changed = false;
	 for( int i = 0; i < intervals.size() -1  ; i ++ ){
	    ia = (Interval)(intervals.get(i));
	    for( int j = i + 1 ; j < intervals.size() ; j ++ ){
	       ib = (Interval)(intervals.get(j));
	       Interval[] iva = ia.commonSubSets( ib );

	       if( iva.length > 0 ){
		  changed = true;
	       }
	       
	       for( int k = 0; k < iva.length; k ++ ){
		  tmpintervals.add( iva[k] );
		  ia.subtractRealSubSet( iva[k] );
		  ib.subtractRealSubSet( iva[k] );
	       }
	    }
	 }
	 for( int i = 0; i < tmpintervals.size() ; i ++ )
	    intervals.add( tmpintervals.get(i));

	 intervals = sortAndRemoveDuplicates( intervals );
      }      

   
      if(printloop){
	 System.out.println(" --- intervals --- ");
	 
	 for( int i = 0; i < intervals.size() ; i ++ ){
	    ((Interval)(intervals.get(i))).setStopNodes();
	    System.out.println( (Interval)(intervals.get(i)) );
	 }
      }

      if(output){
	 System.out.println(intervals.size());
	 for(int i = 0; i < intervals.size() ; i ++ ){
	    Interval iv = (Interval)(intervals.get(i));
	    iv.setStopNodes();
	    Node[] ivmembers = iv.getMembers();
	    System.out.println(ivmembers.length);
	    System.out.println("1.0\n0"); // for continuefile
	    for( int j = 0; j < ivmembers.length ; j ++ ){
	       System.out.println(ivmembers[j].getSignature() + " 0");
	    }
	    Node[] ivstop = iv.getStopAdrs();
	    System.out.println(ivstop.length);
	    for( int j = 0; j < ivstop.length ; j ++ ){
	       System.out.println(ivstop[j].getSignature());
	    }
	 }
      }

   }

   private ArrayList sortAndRemoveDuplicates( ArrayList al ){
      ArrayList ret = new ArrayList();
      while( ! al.isEmpty() ){
	 Interval iv = (Interval)(al.get(0));
	 if( (!ret.contains(iv)) && iv.size() > 0){
	    boolean containsEqual = false;
	    for( int i = 0; i < ret.size() ; i ++ ){
	       if( ((Interval)ret.get(i)).equal(iv))
		  containsEqual = true;
	    }
	    if(!containsEqual)
	       ret.add(al.remove(0));
	    else
	       al.remove(0);
	 }
	 else
	    al.remove(0);
      }
      Interval[] iva = new Interval[ret.size()];
      iva = (Interval[])ret.toArray(iva);
      Arrays.sort(iva);
      ret.clear();
      for( int i = 0; i < iva.length ; i ++ ){
	 ret.add( iva[i] );
      }

      return ret;
   }


   private Node[] getSubSet( Node[] na, int s, int f ){
      if( s < 0 || f >= na.length)
	 return null;
      Node[] sub = new Node[f-s];
      for( ; s < f ; s ++ ){
	 sub[s] = na[s];
      }
      return sub;
   }
   

   public static void main( String[] args ){
      BufferedReader in = new BufferedReader( new InputStreamReader( System.in) );
      int totalnumber=0;
      int[][] adjacency_matrix;
      String[] signatures;
      
      try{
	 totalnumber = Integer.parseInt( in.readLine());
      }
      catch( Exception e ){
	 e.printStackTrace();
      }
      
      adjacency_matrix = new int[totalnumber][totalnumber];
      signatures = new String[totalnumber];

      try{
	 for( int i = 0; i < totalnumber ; i ++ ){
	    signatures[i] = in.readLine();
	 }

	 StringTokenizer st;
	 for( int i = 0; i < totalnumber ; i ++ ){
	    st = new StringTokenizer( in.readLine() );
	    for( int j =0; j < totalnumber ; j ++ ){
	       adjacency_matrix[i][j] = Integer.parseInt( st.nextToken());
	    }
	 }
	 
      }
      catch( Exception e ){
	 e.printStackTrace();
      }
      
      

      Node[] nodelist = new Node[totalnumber];
      for( int i = 0; i < totalnumber ; i ++ ){
	 nodelist[i] = new Node(signatures[i], i);
      }
      
      /*
	Construct the flowgraph
      */
      for( int i = 0; i < totalnumber ; i ++ ){
	 Node tmp = nodelist[i];	 
	 for( int j = 0; j < totalnumber ; j ++ ) {
	    if( adjacency_matrix[i][j] > 0 ){
	       Edge e = new Edge( tmp, nodelist[j] );
	       tmp.addOutEdge( e );
	       nodelist[j].addInEdge( e );
	    }
	 }
      }

      Identify id = new Identify();
      Node start = nodelist[0];
      
      if(verbose)
	 printloop = true;

      if(output){
	 verbose = printloop = iverbose = false;
      }
       

      id.buildDominTree( nodelist, start );
      id.buildDJGraph( nodelist, start );
      
      if(verbose){
	 for( int i = 0 ; i < nodelist.length ; i ++ ){
	    System.out.println( nodelist[i] );
	 }
      }

      id.mainLoop( nodelist );

      /*
	Reset the flowgraph
      */

      Arrays.sort( nodelist , new MnumberComparator() );

      
      for( int i = 0; i < totalnumber ; i ++ ){
	 nodelist[i].setInEdges( new EdgeList() );
	 nodelist[i].setOutEdges( new EdgeList() );
      }
      
      for( int i = 0; i < totalnumber ; i ++ ){
	 Node tmp = nodelist[i];	 
	 for( int j = 0; j < totalnumber ; j ++ ) {
	    if( adjacency_matrix[i][j] > 0 ){
	       Edge e = new Edge( tmp, nodelist[j] );
	       tmp.addOutEdge( e );
	       nodelist[j].addInEdge( e );
	    }
	 }
      }
      
      if(printloop)
	 id.printLoops();

      id.printIntervals();
   }   
}


class Interval implements Comparable{
   Node[] members;
   Node[] stopAdrs;

   public Interval( Node[] members ){
      this.members = members;
   }

   public Interval(){
      members = new Node[0];
   }

   public boolean isEmpty(){
      return members.length == 0;
   }

   public int size(){
      return members.length;
   }   

   public Node[] getMembers(){
      return members;
   }

   public Node[] getStopAdrs(){
      return stopAdrs;
   }

   public Node getTarget(){
      return members[0];
   }

   public boolean isRealSubSetOf( Interval iv ){
      if( iv.size() < members.length ){
	 return false;
      }
      int i;

      Node[] ivmembers = iv.getMembers();
      for( i = 0; i < iv.size() ; i ++ ){
	 if( ivmembers[i] == members[0] )
	    break;
      }

      if( (iv.size() - i) < members.length )
	 return false;

      for( int j = 0; j < members.length ; j ++ , i ++ ){
	 if( members[j] != ivmembers[i] )
	    return false;
      }
      
      return true;
   }

   public Interval[] commonSubSets( Interval iv ){
      Node[] ivmembers = iv.getMembers();

      ArrayList subsets = new ArrayList();

      for( int i = 0; i < members.length ; i ++ ){
	 
	 for( int j = 0; j < ivmembers.length ; j ++ ){
	    System.out.println("mem[i]= "+members[i]+", ivmembers[j]= "+ivmembers[j]);
	    if( members[i] == ivmembers[j] ){
	       ArrayList naal = new ArrayList();
	       while( members[i] == ivmembers[j] ){
		  naal.add( members[i] );
		  i ++;
		  j ++;
		  if( j >= ivmembers.length || i >= members.length )
		     break;
	       }
	       Node[] na = new Node[naal.size()];
	       na = (Node[])naal.toArray(na);
	       subsets.add(na);
	    }
	 }
      }
      
      Interval[] iva = new Interval[subsets.size()];
      for( int i = 0; i < subsets.size() ; i ++ ){
	 iva[i] = new Interval((Node[])(subsets.get(i)));
      }
      return iva;
   }

   public void subtractRealSubSet( Interval iv ){

      Node[] newmembers = new Node[members.length - iv.size()];
      int i,j;
      Node ivtarget = iv.getTarget();
      for( i = 0; i < members.length ; i ++ ){
	 if( members[i] == ivtarget )
	    break;
	 else{
	    newmembers[i] = members[i];
	 }
      }

      for( j= i , i += iv.size() ; i < members.length ; i ++ , j ++ ){
	 newmembers[j] = members[i];
      }

      members = newmembers;
   }

   public boolean equal( Interval iv ){
      if( iv.size() != members.length )
	 return false;
      Node[] othermembers = iv.getMembers();
      for( int i = 0; i < members.length ; i ++ ){
	 if( members[i] != othermembers[i] )
	    return false;
      }
      return true;
   }

   public Interval[] split( Node target ){
      Node[] tmp = (Node[])members.clone();
      int nofsplits = 0;
      int splitidx[] = new int[members.length];
      Interval[] iva = new Interval[1];
      if( target == this.getTarget() ){
	 iva[0] = this;
	 return iva;
      }

      for( int i = 1 ; i < tmp.length ; i ++ ){
	 if( target == tmp[i] ){
	    splitidx[nofsplits ++] = i ;
	 }
      }

      if( nofsplits == 0 ){
	 iva[0] = this;
	 return iva;
      }
      
      ArrayList ivaal = new ArrayList();
  
      for( int i = 0, last = 0; i < nofsplits ; i ++ ){
	 Node[] na = new Node[splitidx[i] - last ];
	 for( int j = last, naidx = 0 ; j < splitidx[i] ; j ++ , naidx ++ ){
	    na[naidx] = tmp[j] ;
	 }
	 last = splitidx[i]+1;
	 if(Identify.iverbose)System.out.println("Creating a new splitted interval : "+(new Interval(na)));
	 ivaal.add(new Interval(na));
      }

      if( splitidx[--nofsplits] < ( tmp.length - 1 ) ) {

	 Node[] na = new Node[(tmp.length -1 ) - (splitidx[nofsplits])];
	 
	 if(Identify.iverbose){

	    System.out.println("splitidx[--nofsplits] = "+splitidx[nofsplits]);
	    System.out.println("tmp.length - 1  = "+(tmp.length-1));
	    System.out.println("na.length = "+(na.length));
	 }

	 for( int i = splitidx[nofsplits]+1, naidx = 0 ; i < tmp.length ; i ++ , naidx ++ ){
	    na[naidx] = tmp[i];
	 }
	 if(Identify.iverbose)System.out.println("The last splitted interval : "+(new Interval(na)));
	 ivaal.add(new Interval(na));
      }

      iva = new Interval[ivaal.size()];
      iva = (Interval[])(ivaal.toArray(iva));
      return iva;
   }

   public void setStopNodes(){
      Node last = members[members.length - 1];
      EdgeList el = last.getOutEdges();
      stopAdrs = new Node[el.size()];
      for( int i = 0; i < el.size() ; i ++ ){
	 stopAdrs[i] = el.get(i).getDst();
      }
   }

   public int compareTo( Object other ){
      Interval it = (Interval)other;
      return members.length - it.size();
   }
   
   public String toString(){
      String out = "[ ";
      for( int i = 0; i < members.length ; i ++ ){
	 out += members[i].getSignature()+" ";
      }
      out += "]";
      if( stopAdrs != null ){

      String stop = "[ ";
      for( int i = 0; i < stopAdrs.length ; i ++ ){
	 stop += stopAdrs[i].getSignature()+" ";
      }
      stop += "]";
      return out + " stopadrs: " + stop;
      }
      return out;
   }

}
