import java.util.ArrayList;
import java.util.Arrays;

public class Node{
   
   private int dominTreeLevel;
   private Node idom;
   private Node pre;
   
   private int fnumber;
   private boolean found;
   
   private EdgeList outEdges;
   private EdgeList inEdges;

   private String signature;

   private ArrayList loops;

   private boolean isContained;
   private boolean doesContain;

   private int mnumber;

   public Node( String signature, int mnumber ){
      this.signature = signature;
      outEdges = new EdgeList();
      inEdges = new EdgeList();
      idom = pre = null;
      loops = new ArrayList();
      found = isContained = false;
      this.mnumber = mnumber;
   }

   public ArrayList getLoops(){
      return loops;
   }

   public int getMnumber(){
      return mnumber;
   }

   public boolean getIsContained(){
      return isContained;
   }

   public void setIsContained( boolean b ){
      isContained = b;
   }

   public boolean doesContain(){
      return doesContain;
   }

   public void setDoesContain( boolean b ){
      doesContain = b;
   }


   public void addOutEdge( Edge e){
      outEdges.add( e );
   }

   public void addInEdge( Edge e ){
      inEdges.add( e );
   }

   public EdgeList getInEdges(){
      return inEdges;
   }

   public EdgeList getOutEdges(){
      return outEdges;
   }

   public boolean hasStrParent( Node n ){
      return inEdges.containsSrc( n );
   }

   public void setDominTreeLevel(){
      dominTreeLevel = 0;
      Node n = this;
      while( n.getIdom() != n ){
	 dominTreeLevel ++;
	 n = n.getIdom();
      }
   }

   private boolean isDominatedBy( Node n ){
      Node m = this;
      if( n == m )
	 return true;
      while( m.getIdom() != m ){
	 if( m.getIdom() == n )
	    return true;
	 m = m.getIdom();
      }
      return false;
   }

   private boolean isChildOf( Node n ){
      Node m = this;
      if( n == m ){
	 return true;
      }
      while( m.getPre() != m ){
	 if( m.getPre() == n )
	    return true;
	 m = m.getPre();
      }
      return false;
   }

   public void setOutEdgeCharacteristics(){
      for( int i = 0; i < outEdges.size() ; i ++ ){
	 Edge e = outEdges.get(i);
	 Node dst = e.getDst();
	 int ch = 0;
	 if( dst.getIdom() == this ){
	    ch |= Edge.edge_Dominator;
	 }
	 else if( this.isDominatedBy( dst )){
	    ch |= Edge.edge_Back_join;
	 }
	 else{
	    ch |= Edge.edge_Cross_join ;
	 }
	 
	 if( dst.getPre() == this ){
	    ch |= Edge.edge_Sp_tree ;
	 }
	 else{
	    if( this.isChildOf( dst ) ){
	       ch |= Edge.edge_Sp_back ;
	    }
	    else if( dst.isChildOf( this ) ){
	       ch |= Edge.edge_Sp_forward;
	    }
	    else{
	       ch |= Edge.edge_Sp_cross;
	    }
	 }
	 e.setCh( ch );
      }
      
   }


   public ArrayList reachUnderSet( Node n ){

      ArrayList found = new ArrayList();
      ArrayList fifo = new ArrayList();

      /*Pointing to self*/
      if( n == this ){
	 found.add( n );
	 return found;
      }
      
      found.add( this );
      fifo.add(this);
      while( ! fifo.isEmpty() ){
	 Node t = (Node)fifo.remove(0);
	 EdgeList el = t.getInEdges();
	 for( int i = 0; i < el.size() ; i ++ ){
	    Node src = el.get(i).getSrc();
	    if( !found.contains( src ) && src != n ){
	       fifo.add( src );
	       found.add( src );
	    }
	 }
      }
      return reverse(found);
   }

   private ArrayList reverse( ArrayList a ){
      ArrayList ret = new ArrayList();
      for( int i = a.size() -1 ; i > -1 ; i -- )
	 ret.add( a.get(i));
      return ret;
   }

   public void setInEdges( EdgeList el ){
      inEdges = el;
   }

   public void setOutEdges( EdgeList el ){
      outEdges = el;
   }
      
   public void addLoop( ArrayList rset ){
      Node[] loop = new Node[rset.size()];
      loop = (Node[])rset.toArray( loop );
      loops.add( loop );
   }

   /*--------------------------------------------------------------*/
   /* Get and set methods */
   /*--------------------------------------------------------------*/
   

   public void setDominTreeLevel( int i ){
      dominTreeLevel = i;
   }

   public int getDominTreeLevel(){
      return dominTreeLevel;
   }

   public void setFnumber( int i ){
      fnumber = i;
   }

   public int getFnumber(){
      return fnumber;
   }

   public void setFound( boolean b){
      found = b;
   }

   public boolean getFound(){
      return found;
   }

   public void setIdom( Node n ){
      idom = n;
   }

   public Node getIdom(){
      return idom;
   }

   public void setPre( Node n ){
      pre = n;
   }

   public Node getPre(){
      return pre;
   }

   public String getSignature(){
      return signature;
   }

   public String toString(){
      String out = "";
      out += "I am "+signature +":\n";
      out += "My idom is "+ (( idom != null ) ? idom.getSignature() : "null");
      out += "\n";
      out += "My pre is "+ (( pre != null ) ? pre.getSignature() : "null" );
      out += "\nMy fnumber is "+fnumber+"\n";
      out += "My domintreelevel is "+ dominTreeLevel+"\n";

      out += "With outgoing edges to:\n";
      for( int i = 0; i < outEdges.size() ; i ++ ){
	 out+= "("+outEdges.get(i).getCh()+") -> ";
	 out+= outEdges.get(i).getDst().getSignature() +" | ";
      }
      out += "\nAnd incoming from:\n";
      
      for( int i = 0; i < inEdges.size() ; i ++ ){
	 out += "("+inEdges.get(i).getCh()+") -> ";
	 out += inEdges.get(i).getSrc().getSignature() +" | ";
      }
      out += "\n";

      String hline = "----------------------\n";
      return hline + out + hline;
   }


   public void printLoops(){


      for( int i = 0; i < loops.size() ; i ++ ){
	 Node[] tmp = (Node[])loops.get(i);
	 Arrays.sort( tmp, new MnumberComparator() );
	 System.out.print("[ " +signature +" ");
	 for( int j = 0; j < tmp.length; j ++ ){
	    if( tmp[j].getLoops().size() > 0 ){
	       System.out.print("[ ");
	       System.out.print( tmp[j].getSignature() +" " );
	       System.out.print("] ");
	    }
	    else
	       System.out.print( tmp[j].getSignature() +" " );

	 }
	 System.out.print(" ]\n");
      }
   }

}
